#!/usr/bin/perl -I/usr/sausalito/perl -I.
# $Id: 17_bx_cloud_init.pl

# this needs use lib because it isn't a handler
use lib qw(/usr/sausalito/handlers/base/network);
use CCE;
use Network;

# Debugging switch:
$DEBUG = "1";
if ($DEBUG) {
        use Data::Dumper;
        use Sys::Syslog qw( :DEFAULT setlogsock);
}

my $errors = 0;

my $cce = new CCE;
$cce->connectuds();

&debug_msg("Running: 17_bx_cloud_init.pl\n");

#
### Information gathering phase (lining up the ducks):
#

# Get 'System' object:
my ($sysoid) = $cce->find('System');
my ($ok, $System) = $cce->get($sysoid);
if (!$ok) {
    debug_msg("Failed to get 'System' object!\n");
    $cce->bye('FAIL');
    exit 1;
}

#
### Special Incus privions (Cloud-Init that works!)
#

my %BX_CLOUD = {};
$bx_cloud_cfg = '/tmp/bx_cloud_cfg';
if ((-e '/dev/incus/sock') && (-f $bx_cloud_cfg)) {
    open my $fh, '<', $bx_cloud_cfg or die "Could not open '$bx_cloud_cfg': $!";

    while (my $line = <$fh>) {
        chomp $line;
        my ($key, $value) = split(/\|/, $line, 2);

        # Check if the key is not empty and the value is defined
        if (defined $key && $key ne '' && defined $value && $value ne '') {
            if ($key eq 'FQDN') {
                my ($hostname, $domainname) = split(/\./, $value, 2);
                $BX_CLOUD->{'hostname'} = $hostname;
                $BX_CLOUD->{'domainname'} = $domainname;
            }
            else {
                $BX_CLOUD->{$key} = $value;
            }
        }
    }
    close $fh;

    if ($BX_CLOUD->{'ipaddr'}) {
        $primary_ipv4 = $BX_CLOUD->{'ipaddr'};
    }

    if ($BX_CLOUD->{'ipaddr_IPv6'}) {
        $primary_ipv6 = $BX_CLOUD->{'ipaddr_IPv6'};
    }
}
else {
        # Done:
        debug_msg("This is not a BlueOnyx running on 6110R. Nothing to do. Exiting.\n");
        $cce->bye('SUCCESS');
        exit(0);
}

#
### Use our 6110R created Cloud-Config to configure the Network:
#

# Obj 'Network' for 'eth0';
my $bx_net_device = 'eth0';
$REALSTATE->{'NIC'}->{$bx_net_device}->{'real'} = 1;
$REALSTATE->{'NIC'}->{$bx_net_device}->{'netmask'} = $BX_CLOUD->{'netmask'};
$REALSTATE->{'NIC'}->{$bx_net_device}->{'bootproto'} = 'none';
if (($BX_CLOUD->{'dhcp4'} eq 'yes') || ($BX_CLOUD->{'dhcp6'} eq 'yes')) {
        $REALSTATE->{'NIC'}->{$bx_net_device}->{'bootproto'} = 'dhcp';
}
$REALSTATE->{'NIC'}->{$bx_net_device}->{'ipaddr_IPv6'} = $BX_CLOUD->{'ipaddr_IPv6'};
$REALSTATE->{'NIC'}->{$bx_net_device}->{'enabled'} = 1;
$REALSTATE->{'NIC'}->{$bx_net_device}->{'ipaddr'} = $BX_CLOUD->{'ipaddr'};
$REALSTATE->{'NIC'}->{$bx_net_device}->{'mac'} = get_device_mac($bx_net_device);

# 'System' object data:
$REALSTATE->{'CONFIG'}->{'dns'} = $BX_CLOUD->{'dns'};
$REALSTATE->{'CONFIG'}->{'gateway'} = $BX_CLOUD->{'gateway'};
$REALSTATE->{'CONFIG'}->{'gateway_IPv6'} = $BX_CLOUD->{'gateway_IPv6'};
$REALSTATE->{'CONFIG'}->{'extra_ipaddr'} = '';
$REALSTATE->{'CONFIG'}->{'extra_ipaddr_IPv6'} = '';
$REALSTATE->{'CONFIG'}->{'hostname'} = $BX_CLOUD->{'hostname'};
$REALSTATE->{'CONFIG'}->{'domainname'} = $BX_CLOUD->{'domainname'};

# Type of network:
my $IPType = 'BOTH';
if ((($BX_CLOUD->{'ipaddr'} != '') && ($BX_CLOUD->{'ipaddr_IPv6'} != '')) || (($BX_CLOUD->{'dhcp4'} eq 'yes') && ($BX_CLOUD->{'dhcp6'} eq 'yes'))) {
        $IPType = 'VZBOTH';
}
elsif ((($BX_CLOUD->{'ipaddr'} != '') && ($BX_CLOUD->{'ipaddr_IPv6'} == '')) || (($BX_CLOUD->{'dhcp4'} eq 'yes') && ($BX_CLOUD->{'dhcp6'} eq 'no'))) {
        $IPType = 'VZv4';
}
elsif ((($BX_CLOUD->{'ipaddr'} == '') && ($BX_CLOUD->{'ipaddr_IPv6'} != '')) || (($BX_CLOUD->{'dhcp4'} eq 'no') && ($BX_CLOUD->{'dhcp6'} eq 'yes'))) {
        $IPType = 'VZv6';
}
else {
        $IPType = 'BOTH';
}

my $primary_ipv6 = get_primary_ipv6_ip();

if ($REALSTATE->{'NIC'}->{$bx_net_device}->{'ipaddr_IPv6'} != $primary_ipv6) {
        if ($primary_ipv6 != '') {
            $ip_rem_line = $primary_ipv6;
            system("/usr/sbin/ip addr del $ip_rem_line dev eth0");
        }
}

# Prepare result for publishing:
$REALSTATE->{'CONFIG'}->{'IPType'} = $IPType;

# Early commit:
my $bx_net_device = 'eth0';
my @net_oid = $cce->find('Network',  { 'device' => $bx_net_device } );
if ($net_oid[0]) {
        debug_msg("Have $bx_net_device in CODB as OID: $net_oid[0]\n");

        # Run $cce->update() which will only perform a SET if the data in CODB is different:
        my ($ok) = $cce->set($net_oid[0], '', $REALSTATE->{'NIC'}->{$bx_net_device});

        # Remove OID of this existing Network device from @Network_OIDs):
        @Network_OIDs = grep { $_ ne $net_oid[0] } @Network_OIDs;
}
else {
        debug_msg("Do not Have $bx_net_device in CODB\n");

        # No Network object for this device in CODB yet. So we create one:
        $REALSTATE->{'NIC'}->{$bx_net_device}->{device} = $bx_net_device;
        $REALSTATE->{'NIC'}->{$bx_net_device}->{refresh} = time();

        my ($success) = $cce->create('Network', $REALSTATE->{'NIC'}->{$bx_net_device});
        if (!$success) {
            debug_msg("Failed to create Network object for $bx_net_device\n");
            $errors++;
        } 
        else {
            debug_msg("Created Network object for $bx_net_device.\n");
        }
        # Turn on NAT and IPForwarding
        hack_on_nat($sysoid);
}

# Update 'System' object:
my ($ok) = $cce->set($sysoid, '', $REALSTATE->{'CONFIG'});
if (!$ok) {
        debug_msg("Updating of 'System' object failed!\n");
}

# Remove $bx_cloud_cfg:
system("rm -f $bx_cloud_cfg");

debug_msg("All done. Exiting.\n");
$cce->bye('SUCCESS');
exit(0);

#
### Subs:
#

# Get device name of primary network interface:
sub get_primary_interface {
    my $primary_interface = '';
    my @routes = `ip route | grep default`;

    foreach my $route (@routes) {
        if ($route !~ /linkdown/ && $route !~ /veth/) {
            ($primary_interface) = $route =~ /\bdev\s+(\S+)/;
            last if $primary_interface;
        }
    }
    chomp($primary_interface);

    if ($primary_interface eq '') {
        $primary_interface = `ifconfig | grep -E '^[a-zA-Z0-9]+' | awk '{print \$1}' | sed 's/://g' | head -1`;
        chomp($primary_interface);
    }

    return $primary_interface;
}

sub get_primary_ipv6_ip {
    my ($actual_device) = @_;

    # If no device is provided, use the primary network interface
    $actual_device //= get_primary_interface();

    # Get IPv6 IP:
    my $ipv6_ip = `LC_ALL=C ip -6 addr show dev $actual_device|grep inet6|awk '{ print \$2 }'|cut -d / -f1|grep -v '^fe80::'|head -1`;
    chomp($ipv6_ip);

    return $ipv6_ip;
}

# Get MAC address of specified network device:
sub get_device_mac {
    my ($actual_device) = @_;

    # If no device is provided, use the primary network interface
    $actual_device //= get_primary_interface();

    # Get MAC:
    my $mac = `/usr/bin/nmcli device show $actual_device|grep GENERAL.HWADDR:|awk {'print \$2'}`;
    chomp($mac);

    return $mac;
}

sub hack_on_nat {
    my ($oid) = @_;
    if ($oid ne '') {
        # Get 'System' / 'Network'
        my ($ok, $System_Network) = $cce->get($oid, 'Network');

        if (($System_Network->{nat} ne '1') || ($System_Network->{ipForwarding} ne '1')) {
            my ($ok) = $cce->update($oid, 'Network', { 'nat' => '1', 'ipForwarding' => '1' });
        }
    }
}

sub debug_msg {
    my $msg = shift;
    if ($DEBUG gt '1') {
        print "$msg";
    }
    if ($DEBUG gt '0') {
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$msg");
        closelog;
    }
}

# 
# Copyright (c) 2008-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 